*! version 1.0.6  06feb2006  by Marc Muendler muendler@ucsd.edu

clear
set memory 400m
set matsize 400
set more off
capture log close

cd "$rootslct"

log using "logs/probit-selct-$waglvl-pt$par-af$aff", replace

capture mkdir "$rootslct/estmat/v3/pt$par-af$aff-$waglvl-cf"
capture mkdir "$rootslct/estmat/v3/pt$par-af$aff-$waglvl-cf/probit"

mat drop _all

local maxiter $maxiter
local usexvar $usexvar  /* probit varlist to be used for selection correction in sureg */
local maxvars 4  /* longest possible probit varlist */
local surxno 15  /* number of SURE regressors per equation (excluding noFDI indicators, excluding adj. constant/ones and mills ratio) */

adopath + "$root/%stata/ado"

#delimit ;


/* variable sets for probit estimation */
/*-------------------------------------*/

do $dp/!varsets.do;

/* data */
/*------*/

tempfile popinfo;
use "$rootslct/Xdata/pop-reg-nonattrition-pt$par-af$aff", clear;
keep if year==2000;
keep mneid i*;  /* do not update sample-specifi nofdi indicators for translog */
sort mneid;
compress;
save `popinfo';

foreach usemissg in $howmany {; /* begin `usemissg' loop */

use "$rootslct/Xdata/mneid-outwfdi-ustan-2000-translog-$waglvl-pt$par-af$aff", clear;
drop naff;

foreach var in s w t k {;
  gen double cf_`var' = cf * `var';
  };
drop l;
replace cntrybemgrp=(substr(cntrybemgrp,1,2));
rename cf_s x;

if `usemissg' == 0 {;
* keep usemissg observations as they are;
    drop nofdi;
    /* s:costShare x:Labor w:Wage t:Turnover("Output") k:Capital */
    reshape wide s w t k x cf_w cf_t cf_k cf, i(mneid) j(cntrybemgrp) string;
    };

if `usemissg' == 1 {;
* construct stacked system;
    foreach var in s w t k x cf_w cf_t cf_k cf {;
        replace `var'=0 if `var'==.;
       };
    reshape wide s w t k x cf_w cf_t cf_k cf nofdi, i(mneid) j(cntrybemgrp) string;
    };

sort mneid;
merge mneid using "$rootslct/Xdata/mvprobit-outwfdi-ustan-2000-$waglvl-pt$par-af$aff";
tab _merge;
drop _merge;
sort mneid;

* competitor regressors;
merge mneid using "$rootslct/Xdata/mneid-outwfdi-ustan-1996-costothr-$waglvl-pt$par-af$aff";
tab _merge;
foreach var in s k t l w wb {;
  foreach reg in $regagg {;
    qui gen _avg_`var'_`reg' = `var'_othr_`reg' if _m==2; /* market average (mneid==0 in using data) */
    qui egen _rpl_`var'_`reg' = max(_avg_`var'_`reg');
    qui replace `var'_othr_`reg' = _rpl_`var'_`reg' if `var'_othr_`reg'==.; /* replace missing competitor info with market average (mneid==0 in using data) */
    drop _avg_`var'_`reg' _rpl_`var'_`reg';
    };
  };
drop if mneid==0;
drop _merge;
sort mneid;
merge mneid using "$rootslct/Xdata/mneid-outwfdi-ustan-1996-wwlocvars-pt$par-af$aff";
* following code only used when "ustan-1996-locvars" loaded (not "-wwlocvars")
* foreach reg in $regagg {;
*   local r2dg = substr("`reg'",1,2);
*   foreach var in employmt ln_dist ln_gdp ln_gdp_cap ln_consum_exp ln_cons_cap lh99_1 lh99_2 {; 
*     qui gen _avg_`var' = `var'_othr_`reg' if _m==2; /* market average (mneid==0 in using data) */
*     qui egen _rpl_`var' = max(_avg_`var');
*     qui replace `var'_othr_`reg' = _rpl_`var' if `var'_othr_`reg'==.;  /* replace missing competitor info with market average (mneid==0 in using data) */
*     rename `var'_othr_`reg' `var'_othr_`r2dg'; 
*     drop _avg_`var' _rpl_`var';
*     };
*   };
drop if mneid==0;
tab _merge;
drop _merge;
sort mneid;

* parent level regressors;
foreach reg in $regagg {;
  local r2dg = substr("`reg'",1,2);
  foreach year in 1996 2000 {;
    local yr = substr("`year'",-2,2);
    drop chosen_`reg'_`year';  /* drop sample presence indicators, use population indicators below */
    rename ln_wagediff_`reg'_`year' ln_wdiff_`r2dg'`yr';
    };
  };
foreach year in 1996 2000 {;
  local yr = substr("`year'",-2,2);
  rename ln_inter_`year' ln_inter`yr';
  foreach var in ln_employmt ln_exports ln_fixdasst ln_assettot ln_profits ln_turnover ln_equity ln_liab ln_caplabrat profeqtyrat {;
    rename `var'_par_`year' `var'`yr';
    };
  };
drop w_HOM_2000; rename w_HOM_1996 w_HOM;  /* use 1998 BA wages (1996 proxied with 1998) */
replace w_HOM = exp(w_HOM);  /* plain wage */
local lbl : variable label w_HOM;
local lbl = subinstr("`lbl'","ln ","",.);
label var w_HOM "`lbl'";

* wage variable redefinitions: all wages in logs;
replace w_HOM = log(w_HOM);
local lbl : variable label w_othr_HOM;
local lbl = "ln `lbl'";
label var w_HOM "`lbl'";
foreach reg in $regagg {;
  replace w_othr_`reg' = log(w_othr_`reg');
  local lbl : variable label w_othr_`reg';
  local lbl = "ln `lbl'";
  label var w_othr_`reg' "`lbl'";
  };

* regional presence indicators and affiliate counts from population information;
gen byte iHO00 = 1;  /* Permanent presence in Home */
label var iHO00 "FDI presence HOM 00";
sort mneid;
merge mneid using `popinfo', nokeep;
tab _m;
drop _m;


/* multivariate probit estimation */
/*--------------------------------*/

forvalues xvars = `maxvars'(-1)0 {;
  * p-value of test that all eta==0: chiprob(`e(nrho)', e(chi2_c));
  di;
  xi: mvprobit (iCE00: iCE00 = iCE96 iDE96 iOI96 iWE96 ${par_`xvars'} i.sec1) (iDE00 : iDE00 = iCE96 iDE96 iOI96 iWE96 ${par_`xvars'} i.sec1) (iOI00: iOI00 = iCE96 iDE96 iOI96 iWE96 ${par_`xvars'} i.sec1) (iWE00 : iWE00 = iCE96 iDE96 iOI96 iWE96 ${par_`xvars'} i.sec1), robust iterate(`maxiter');
    capture local chk = rowsof(mvprobit_b);  /* check whether matrix exists to add to */
    if _rc==111 {;
      est2vec mvprobit, e(N ll df_m chi2_c nrho) name(mvp`xvars'i);
      };
    else {;
      est2vec mvprobit, addto(mvprobit) name(mvp`xvars'i);
      };
  mvprobit (iCE00: iCE00 = iCE96 iDE96 iOI96 iWE96 ${par_`xvars'}) (iDE00 : iDE00 = iCE96 iDE96 iOI96 iWE96 ${par_`xvars'}) (iOI00: iOI00 = iCE96 iDE96 iOI96 iWE96 ${par_`xvars'}) (iWE00 : iWE00 = iCE96 iDE96 iOI96 iWE96 ${par_`xvars'}), robust iterate(`maxiter');
  est2vec mvprobit, addto(mvprobit) name(mvp`xvars');
  };


/* single-equation probit estimation */
/*-----------------------------------*/

foreach reg in $regagg {;
  forvalues xvars = `maxvars'(-1)0 {;
    local r2dg = substr("`reg'",1,2);
    tempvar pxb;
    tempname chk;

    * probit regressions with sector dummies;
    xi: dprobit i`r2dg'00 iCE96 iDE96 iOI96 iWE96 ${par_`xvars'} i.sec1, asis robust iterate(`maxiter');
    capture local chk = rowsof(dprobit`r2dg'_b);  /* check whether matrix exists to add to */
    if _rc==111 {;
      est2vec dprobit`r2dg', vars(iCE96 iDE96 iOI96 iWE96 ${par_`maxvars'} _Isec1_bem_1 _Isec1_bem_2 _Isec1_bem_3 _Isec1_bem_4 _Isec1_bem_5 _Isec1_bem_6 _Isec1_bem_7 _Isec1_bem_8 _Isec1_bem_9 _cons) e(N ll chi2 df_m r2_p pbar xbar) name(`r2dg'`xvars'i) altvec(dfdx se_dfdx);
      };
    else {;
      est2vec dprobit`r2dg', addto(dprobit`r2dg') name(`r2dg'`xvars'i) altvec(dfdx se_dfdx);
      };
    qui probit;  /* report all probit regressions here, only one `xvars' reported with Heckman output */
    capture local chk = rowsof(probit`r2dg'_b);
    if _rc==111 {;
      est2vec probit`r2dg', vars(iCE96 iDE96 iOI96 iWE96 ${par_`maxvars'} _Isec1_bem_1 _Isec1_bem_2 _Isec1_bem_3 _Isec1_bem_4 _Isec1_bem_5 _Isec1_bem_6 _Isec1_bem_7 _Isec1_bem_8 _Isec1_bem_9 _cons) e(N ll chi2 df_m r2_p) name(`r2dg'`xvars'i);
      est2vec probit`r2dg', r(entry_chi2 entry_df entry_p exit_chi2 exit_df exit_p hyst_chi2 hyst_df hyst_p) name(`r2dg'`xvars'i);
      };
    else {;
      est2vec probit`r2dg', addto(probit`r2dg') name(`r2dg'`xvars'i);
      est2vec probit`r2dg', raddto(probit`r2dg') name(`r2dg'`xvars'i);
      };

    * probit regressions with sector and laender dummies;
    xi: dprobit i`r2dg'00 iCE96 iDE96 iOI96 iWE96 ${par_`xvars'} i.sec1, asis robust iterate(`maxiter');
    est2vec dprobit`r2dg', addto(dprobit`r2dg') name(`r2dg'`xvars'iw) altvec(dfdx se_dfdx);
    qui probit;  /* report all probit regressions here, only one `xvars' reported with Heckman output */
    est2vec probit`r2dg', addto(probit`r2dg') name(`r2dg'`xvars'iw);
    test - _cons = 0; 
    est2vec r1, r(chi2 df p) name(`r2dg'`xvars'iw);
    test i`r2dg'96 + _cons = 0; 
    est2vec r2, r(chi2 df p) name(`r2dg'`xvars'iw);
    test - i`r2dg'96 = 0; 
    est2vec r3, r(chi2 df p) name(`r2dg'`xvars'iw);
    mat probit`r2dg'_r = probit`r2dg'_r , (r1_r \ r2_r \ r3_r);
    mat drop r1_r r2_r r3_r;

    * probit regressions without sector dummies;
    dprobit i`r2dg'00 iCE96 iDE96 iOI96 iWE96 ${par_`xvars'}, asis robust iterate(`maxiter');
    est2vec dprobit`r2dg', addto(dprobit`r2dg') name(`r2dg'`xvars') altvec(dfdx se_dfdx);
    qui probit;  /* report all probit regressions here, only one `xvars' reported with Heckman output */
    est2vec probit`r2dg', addto(probit`r2dg') name(`r2dg'`xvars');
    test - _cons = 0; 
    est2vec r1, r(chi2 df p) name(`r2dg'`xvars');
    test i`r2dg'96 + _cons = 0; 
    est2vec r2, r(chi2 df p) name(`r2dg'`xvars');
    test - i`r2dg'96 = 0; 
    est2vec r3, r(chi2 df p) name(`r2dg'`xvars');
    mat probit`r2dg'_r = probit`r2dg'_r , (r1_r \ r2_r \ r3_r);
    mat drop r1_r r2_r r3_r;

    };
  };


/* matrix output */
/*---------------*/

foreach type in b se e {;
  drop _all;
  matsave mvprobit_`type', p(estmat/v3/pt$par-af$aff-$waglvl-cf/probit) t(double) replace;
  };
foreach reg in $regagg {;
  local r2dg = substr("`reg'",1,2);
  foreach type in b se e {;
    drop _all;
    matsave dprobit`r2dg'_`type', p(estmat/v3/pt$par-af$aff-$waglvl-cf/probit) t(double) replace;
    drop _all;
    matsave probit`r2dg'_`type', p(estmat/v3/pt$par-af$aff-$waglvl-cf/probit) t(double) replace;
    };
  drop _all;
  matsave probit`r2dg'_r, p(estmat/v3/pt$par-af$aff-$waglvl-cf/probit) t(double) replace;
  };

* clean up;
drop _all;
foreach type in b se e r {;
  capture mat drop mvprobit_`type';
  foreach reg in $regagg {;
    local r2dg = substr("`reg'",1,2);
    capture mat drop dprobit`r2dg'_`type';
    capture mat drop probit`r2dg'_`type';
    };
  };

}; /* end of `usemissg' loop */

log close;
cd "$root";
