*! version 1.0.1  29sep2005  by Jennifer Poole jppoole@ucsd.edu
*! version 1.0.7  14nov2006  by Marc Muendler muendler@ucsd.edu

/* Varcov correction assumes that selection equations have constant */
/* Selection equation assumed to have constant as last regressor */
/* Varcov correction assumes that sureg equations do not have constant (add a regressor called ones if constant needed) */
/* Sure equation assumed to have Mills ratio as last regressor */
/* Regional wages are supposed to be first (5) regressors in SUR equations */

drop _all
set memory 400m
set matsize 400
set more off
capture log close

cd "$rootslct"

adopath + "$rootslct/%stata/ado"  /* multiheck_d0.ado and multiheck_d1.ado */

log using "logs/translog-selct-2stp-$waglvl-pt$par-af$aff-cf-multiheck.log", replace

mat drop _all

local maxiter $maxiter
local usexvar $usexvar  /* probit varlist to be used for selection correction in sureg */
local maxvars 3  /* longest possible probit varlist */
adopath + "$root/%stata/ado"

#delimit ;


/* variable sets for probit estimation */
/*-------------------------------------*/

do $dp/!varsets.do;

/* define share equation restrictions */
/*------------------------------------*/

* constraint define  1 [sCE]cf_wHO = [sHO]cf_wCE;
  constraint define  2 [sCE]cf_wDE = [sDE]cf_wCE;
  constraint define  3 [sCE]cf_wOI = [sOI]cf_wCE;
  constraint define  4 [sCE]cf_wWE = [sWE]cf_wCE;
* constraint define  5 [sDE]cf_wHO = [sHO]cf_wDE;
  constraint define  6 [sDE]cf_wOI = [sOI]cf_wDE;
  constraint define  7 [sDE]cf_wWE = [sWE]cf_wDE;
* constraint define  8 [sOI]cf_wHO = [sHO]cf_wOI;
  constraint define  9 [sOI]cf_wWE = [sWE]cf_wOI;
* constraint define 10 [sWE]cf_wHO = [sHO]cf_wWE;
* constraint define 11 [sHO]cf_wHO + [sHO]cf_wCE + [sHO]cf_wDE + [sHO]cf_wOI + [sHO]cf_wWE = 0;
  constraint define 12 [sCE]cf_wHO + [sCE]cf_wCE + [sCE]cf_wDE + [sCE]cf_wOI + [sCE]cf_wWE = 0;
  constraint define 13 [sDE]cf_wHO + [sDE]cf_wCE + [sDE]cf_wDE + [sDE]cf_wOI + [sDE]cf_wWE = 0;
  constraint define 14 [sOI]cf_wHO + [sOI]cf_wCE + [sOI]cf_wDE + [sOI]cf_wOI + [sOI]cf_wWE = 0;
  constraint define 15 [sWE]cf_wHO + [sWE]cf_wCE + [sWE]cf_wDE + [sWE]cf_wOI + [sWE]cf_wWE = 0;

/* With 5 regions, there are (5*5-5)/2=10 symmetry restrictions in principle. 
   However, one equation drops due to further cross-equation constraints, 
   only 6 of the symmetry restrictions are needed.
   Similarly, only 4 of the 5 restrictions on wage parameter sums are needed. */


/* data */
/*------*/

local howmany = subinstr("$howmany","0","",.);  /* omnipresent MNE no permitted */
foreach usemissg in `howmany' {; /* begin `usemissg' loop */

capture mkdir "$rootslct/tex/v3ml";
capture mkdir "$rootslct/estmat/v3ml";
capture mkdir "$rootslct/graphs";
capture mkdir "$rootslct/graphs/v3ml";
capture mkdir "$rootslct/tex/v3ml/pt$par-af$aff-$waglvl-cf";
capture mkdir "$rootslct/estmat/v3ml/pt$par-af$aff-$waglvl-cf";
capture mkdir "$rootslct/estmat/v3ml/pt$par-af$aff-$waglvl-cf/multiheck`usemissg'";


/* load matrices of starting values */
/*----------------------------------*/

matload translog`usemissg'_b, p(estmat/v3/pt$par-af$aff-$waglvl-cf/translog`usemissg') overwrite dropall missing(.);
local i = -1;
foreach reg in $regagg {;
  local i = `i' + 2;
  local r2dg = substr("`reg'",1,2);
  matload probit`r2dg'_b, p(estmat/v3/pt$par-af$aff-$waglvl-cf/byreg) overwrite dropall missing(.);
  matrix roweq probit`r2dg'_b = p`r2dg';
  matload sure2stp`r2dg'_e, p(estmat/v3/pt$par-af$aff-$waglvl-cf/byreg) overwrite dropall missing(.);
  if `usemissg'~=2  local reglist "s`r2dg':cf_wHO s`r2dg':cf_wCE s`r2dg':cf_wDE s`r2dg':cf_wOI s`r2dg':cf_wWE s`r2dg':cf_tHO s`r2dg':cf_tCE s`r2dg':cf_tDE s`r2dg':cf_tOI s`r2dg':cf_tWE s`r2dg':cf_kHO s`r2dg':cf_kCE s`r2dg':cf_kDE s`r2dg':cf_kOI s`r2dg':cf_kWE s`r2dg':nofdiCE s`r2dg':nofdiDE s`r2dg':nofdiOI s`r2dg':nofdiWE s`r2dg':cf`r2dg'";
  if `usemissg'==2  local reglist "s`r2dg':cf_wHO_eq`r2dg' s`r2dg':cf_wCE_eq`r2dg' s`r2dg':cf_wDE_eq`r2dg' s`r2dg':cf_wOI_eq`r2dg' s`r2dg':cf_wWE_eq`r2dg' s`r2dg':cf_tHO_eq`r2dg' s`r2dg':cf_tCE_eq`r2dg' s`r2dg':cf_tDE_eq`r2dg' s`r2dg':cf_tOI_eq`r2dg' s`r2dg':cf_tWE_eq`r2dg' s`r2dg':cf_kHO_eq`r2dg' s`r2dg':cf_kCE_eq`r2dg' s`r2dg':cf_kDE_eq`r2dg' s`r2dg':cf_kOI_eq`r2dg' s`r2dg':cf_kWE_eq`r2dg' s`r2dg':nofdiCE_eq`r2dg' s`r2dg':nofdiDE_eq`r2dg' s`r2dg':nofdiOI_eq`r2dg' s`r2dg':nofdiWE_eq`r2dg' s`r2dg':cf`r2dg'_eq`r2dg'";
  local remvar "s`r2dg':nofdi`r2dg'_eq`r2dg'";
  local reglist : list reglist - remvar;
  matorder translog`usemissg'_b, m(sh`r2dg') ifr(`reglist');
  matorder probit`r2dg'_b, m(pr`r2dg') ifc(`r2dg'`usexvar');
  local arho`i' = atanh(sure2stp`r2dg'_e[rownumb(sure2stp`r2dg'_e,"e(rho)"),1]);
  local lns`i' = ln(sure2stp`r2dg'_e[rownumb(sure2stp`r2dg'_e,"e(sigma)"),1]);
  mat drop probit`r2dg'_b sure2stp`r2dg'_e;
  };
mat drop translog`usemissg'_b;

/* multivariate heckman mle without cross equation correlations */
/*--------------------------------------------------------------*/

use "$rootslct/Xdata/mneid-forbstrap-$waglvl-pt$par-af$aff-cf", clear;
drop mCE dltaCE mDE dltaDE mOI dltaOI mWE dltaWE dltaHO;

local surxno : word count cf_wHO cf_wCE cf_wDE cf_wOI cf_wWE cf_tHO cf_tCE cf_tDE cf_tOI cf_tWE cf_kHO cf_kCE cf_kDE cf_kOI cf_kWE cfWE;
local selno : word count iCE96 iDE96 iOI96 iWE96 ${par_`usexvar'};
local surxselno = `surxno' + `selno';  /* number of selection plus SURE regressors per two equations (excluding noFDI indicators, excluding adj. constant/ones and mills ratio) */

if `usemissg'~=2  ml model d1 multiheck_d1
    (sCE: yCE = cf_wHO cf_wCE cf_wDE cf_wOI cf_wWE cf_tHO cf_tCE cf_tDE cf_tOI cf_tWE cf_kHO cf_kCE cf_kDE cf_kOI cf_kWE nofdiDE nofdiOI nofdiWE cfCE, nocons)
    (pCE: iCE00 = iCE96 iDE96 iOI96 iWE96 ${par_`usexvar'})
    (sDE: yDE = cf_wHO cf_wCE cf_wDE cf_wOI cf_wWE cf_tHO cf_tCE cf_tDE cf_tOI cf_tWE cf_kHO cf_kCE cf_kDE cf_kOI cf_kWE nofdiCE nofdiOI nofdiWE cfDE, nocons)
    (pDE: iDE00 = iCE96 iDE96 iOI96 iWE96 ${par_`usexvar'}) 
    (sOI: yOI = cf_wHO cf_wCE cf_wDE cf_wOI cf_wWE cf_tHO cf_tCE cf_tDE cf_tOI cf_tWE cf_kHO cf_kCE cf_kDE cf_kOI cf_kWE nofdiCE nofdiDE nofdiWE cfOI, nocons)
    (pOI: iOI00 = iCE96 iDE96 iOI96 iWE96 ${par_`usexvar'})
    (sWE: yWE = cf_wHO cf_wCE cf_wDE cf_wOI cf_wWE cf_tHO cf_tCE cf_tDE cf_tOI cf_tWE cf_kHO cf_kCE cf_kDE cf_kOI cf_kWE nofdiCE nofdiDE nofdiOI cfWE, nocons)
    (pWE: iWE00 = iCE96 iDE96 iOI96 iWE96 ${par_`usexvar'})
    (lnsigma1_1: )
    (lnsigma3_3: )
    (lnsigma5_5: )
    (lnsigma7_7: ) 
    (athrho1_2: )
    (athrho3_4: )
    (athrho5_6: )
    (athrho7_8: ),
    constraints(2 3 4 6 7 9 12 13 14 15) technique(bhhh);    

if `usemissg'==2  ml model d1 multiheck_d1
    (sCE: yCE = cf_wHO_eqCE cf_wCE_eqCE cf_wDE_eqCE cf_wOI_eqCE cf_wWE_eqCE cf_tHO_eqCE cf_tCE_eqCE cf_tDE_eqCE cf_tOI_eqCE cf_tWE_eqCE cf_kHO_eqCE cf_kCE_eqCE cf_kDE_eqCE cf_kOI_eqCE cf_kWE_eqCE nofdiDE_eqCE nofdiOI_eqCE nofdiWE_eqCE cfCE_eqCE, nocons)
    (pCE: iCE00 = iCE96 iDE96 iOI96 iWE96 ${par_`usexvar'})
    (sDE: yDE = cf_wHO_eqDE cf_wCE_eqDE cf_wDE_eqDE cf_wOI_eqDE cf_wWE_eqDE cf_tHO_eqDE cf_tCE_eqDE cf_tDE_eqDE cf_tOI_eqDE cf_tWE_eqDE cf_kHO_eqDE cf_kCE_eqDE cf_kDE_eqDE cf_kOI_eqDE cf_kWE_eqDE nofdiCE_eqDE nofdiOI_eqDE nofdiWE_eqDE cfDE_eqDE, nocons)
    (pDE: iDE00 = iCE96 iDE96 iOI96 iWE96 ${par_`usexvar'}) 
    (sOI: yOI = cf_wHO_eqOI cf_wCE_eqOI cf_wDE_eqOI cf_wOI_eqOI cf_wWE_eqOI cf_tHO_eqOI cf_tCE_eqOI cf_tDE_eqOI cf_tOI_eqOI cf_tWE_eqOI cf_kHO_eqOI cf_kCE_eqOI cf_kDE_eqOI cf_kOI_eqOI cf_kWE_eqOI nofdiCE_eqOI nofdiDE_eqOI nofdiWE_eqOI cfOI_eqOI, nocons)
    (pOI: iOI00 = iCE96 iDE96 iOI96 iWE96 ${par_`usexvar'})
    (sWE: yWE = cf_wHO_eqWE cf_wCE_eqWE cf_wDE_eqWE cf_wOI_eqWE cf_wWE_eqWE cf_tHO_eqWE cf_tCE_eqWE cf_tDE_eqWE cf_tOI_eqWE cf_tWE_eqWE cf_kHO_eqWE cf_kCE_eqWE cf_kDE_eqWE cf_kOI_eqWE cf_kWE_eqWE nofdiCE_eqWE nofdiDE_eqWE nofdiOI_eqWE cfWE_eqWE, nocons)
    (pWE: iWE00 = iCE96 iDE96 iOI96 iWE96 ${par_`usexvar'})
    (lnsigma1_1: )
    (lnsigma3_3: )
    (lnsigma5_5: )
    (lnsigma7_7: ) 
    (athrho1_2: )
    (athrho3_4: )
    (athrho5_6: )
    (athrho7_8: ),
    constraints(2 3 4 6 7 9 12 13 14 15) technique(bhhh);    

* set starting values;
ml init shCE prCE shDE prDE shOI prOI shWE prWE, skip;
ml init lnsigma1_1:_cons=`lns1';
ml init lnsigma3_3:_cons=`lns3';
ml init lnsigma5_5:_cons=`lns5';
ml init lnsigma7_7:_cons=`lns7';
ml init athrho1_2:_cons=`arho1';
ml init athrho3_4:_cons=`arho3';
ml init athrho5_6:_cons=`arho5';
ml init athrho7_8:_cons=`arho7';

ml maximize, difficult iterate(`maxiter');

est2vec multiheck`usemissg', e(N k_eq) name(multiheck_`usemissg');

ml graph, saving("$rootslct/graphs/v3ml/mlgraph.gph", replace);

* build matrix prbdlnw of probit wage coefficients;
foreach type in b se {;  /* Build up prbdlnw columns in order of $regagg */
  foreach reg in $regagg {;
    local r2dg = substr("`reg'",1,2);
    tempname plnw`r2dg';
    matins multiheck`usemissg'_`type' , m(`plnw`r2dg'') ifr(p`r2dg':w_HOM p`r2dg':w_othr_CEE p`r2dg':w_othr_DEV p`r2dg':w_othr_OIN p`r2dg':w_othr_WEU);
    matrix colnames `plnw`r2dg'' = :`r2dg'`usexvar';
    };
  local cols : word count $regagg;
  tokenize $regagg;
  local j 1;
  local `j' = substr("``j''",1,2);
  mat prbdlnw_`type' = `plnw``j''';
  forvalues j = 2/`cols' {;
    local `j' = substr("``j''",1,2);
    mat prbdlnw_`type' = prbdlnw_`type' , `plnw``j''';
    };
  matrix colnames prbdlnw_`type' = $regagg;
  matrix rownames prbdlnw_`type' = w_HOM w_othr_CEE w_othr_DEV w_othr_OIN w_othr_WEU;
  };
* do not replace missing and insignificant with 0;
* local regj = 0;
* foreach reg in $regagg {;
*   local regj = `regj' + 1;
*   local wi = 0;
*   foreach weff in HOM $regagg {;
*     local wi = `wi' + 1;
*     if prbdlnw_b[`wi',`regj'] == -999 | prbdlnw_b[`wi',`regj'] == . | prbdlnw_se[`wi',`regj'] == -999 | prbdlnw_se[`wi',`regj'] == . | abs(prbdlnw_b[`wi',`regj']/prbdlnw_se[`wi',`regj']) < abs(invnorm((100-${S_level})/200)) {;
*       mat prbdlnw_b[`wi',`regj'] = 0;
*       };
*     };
*   };
mat prbdlnw = prbdlnw_b;
mat drop prbdlnw_b prbdlnw_se;
mat prbdlnw = prbdlnw '; /* regions in rows, wage effects in columns */
local rownames : rowfullnames prbdlnw;
local colnames : colfullnames prbdlnw;
local rownum : word count `rownames';
local colnum : word count `colnames';
tokenize `rownames';
forvalues rowi = 1/`rownum' {;
  local rownew = "i_" + substr("``rowi''",-3,3);
  matname prbdlnw `rownew', rows(`rowi') explicit;
  };
tokenize `colnames';
forvalues coli = 1/`colnum' {;
  local colnew = "w_" + substr("``coli''",-3,3);
  matname prbdlnw `colnew', columns(`coli') explicit;
  };
disp in gr "Matrix of (significant) wage effects on FDI presence (usemissg: " in ye `usemissg' in gr ")";
mat list prbdlnw;


/* initial concavity checks */
/*--------------------------*/

* matrix to check (i) that the fitted cost shares sLOC_predicted are nonnegative;
mat negxhatcnt`usemissg'_ml = J(2,4,.);
matname negxhatcnt`usemissg'_ml r(N_neg_xhare) r(N), rows(1..2) explicit;
local i = 0;
foreach sreg in sCE sDE sOI sWE {;
  local i = `i' + 1;
  matname negxhatcnt`usemissg'_ml `sreg', columns(`i') explicit;
  predict _`sreg' if e(sample), equation(`sreg') xb;
  qui count if _`sreg'<0 & e(sample);
  mat negxhatcnt`usemissg'_ml[1,`i']=`r(N)';
  qui count if e(sample);
  mat negxhatcnt`usemissg'_ml[2,`i']=`r(N)';
  };
drop _s*;


/* sample statistics for elasticities of substitution */
/*----------------------------------------------------*/

order mneid nreg sec1_bem i*;
summ2tex * if e(sample), c(mean sd N p50) name(summ_translog`usemissg'_ml) p(tex/v3ml/pt$par-af$aff-$waglvl-cf) replace detail;
preserve;
  foreach var in sHO sCE sDE sOI sWE wHO wCE wDE wOI wWE tHO tCE tDE tOI tWE kHO kCE kDE kOI kWE cfHO cfCE cfDE cfOI cfWE {;
    replace `var' = . if `var'==0;
    };
  summ2tex * if e(sample), c(mean sd N p50) name(summ_translog`usemissg'_missg_ml) p(tex/v3ml/pt$par-af$aff-$waglvl-cf) replace detail;
  foreach reg in $regagg {;
    local r2dg = substr("`reg'",1,2);
    summ2tex * if e(sample) & nofdi`r2dg'==0, c(mean sd N p50) name(summ_translog`usemissg'_missg`r2dg'_ml) p(tex/v3ml/pt$par-af$aff-$waglvl-cf) replace detail;
    };
restore;
corr2tex sHO sCE sDE sOI sWE if e(sample), name(corrT_translog`usemissg'_ml) p(tex/v3ml/pt$par-af$aff-$waglvl-cf) replace;


/* elasticities of substitution */
/*------------------------------*/

* Cross-wage coefficient matrix;
tempname VCE;
tempname bcoef bse coefmat0 semat0 coefmat1 semat1 coefmat2 semat2 coefmat3 semat3 coefmat4 semat4; 
mat `bcoef' = e(b)';
mat `VCE' = e(V);
mat `bse' = vecdiag(`VCE')';
foreach mattyp in coef se {;
  mat ``mattyp'mat0'_ml = J(5,1,.);
  local cnt 0;
  foreach reg in $regagg {;
    local cnt = `cnt' + 1;
    local r2dg = substr("`reg'",1,2);
    if `usemissg'~=2  matorder `b`mattyp'', m(``mattyp'mat`cnt''_ml) ifr(s`r2dg':cf_wHO s`r2dg':cf_wCE s`r2dg':cf_wDE s`r2dg':cf_wOI s`r2dg':cf_wWE);
    if `usemissg'==2  matorder `b`mattyp'', m(``mattyp'mat`cnt''_ml) ifr(s`r2dg':cf_wHO_eq`r2dg' s`r2dg':cf_wCE_eq`r2dg' s`r2dg':cf_wDE_eq`r2dg' s`r2dg':cf_wOI_eq`r2dg' s`r2dg':cf_wWE_eq`r2dg');
    local col`cnt' : roweq ``mattyp'mat`cnt''_ml; local col`cnt' = substr("`col`cnt''",1,3);
    };
  * mat list ``mattyp'mat1'_ml;
  * mat list ``mattyp'mat2'_ml;
  * mat list ``mattyp'mat3'_ml;
  * mat list ``mattyp'mat4'_ml;
  mat `mattyp'mat_ml = ``mattyp'mat0'_ml , ``mattyp'mat1'_ml , ``mattyp'mat2'_ml , ``mattyp'mat3'_ml , ``mattyp'mat4'_ml;
  * mat list `mattyp'mat_ml;
  mat ``mattyp'mat0'_ml = `mattyp'mat_ml[1,1..5] ';
  mat `mattyp'mat_ml = ``mattyp'mat0'_ml , ``mattyp'mat1'_ml , ``mattyp'mat2'_ml , ``mattyp'mat3'_ml , ``mattyp'mat4'_ml;
  mat `mattyp'mat_ml[1,1] = - (`mattyp'mat_ml[2,1] + `mattyp'mat_ml[3,1] + `mattyp'mat_ml[4,1] + `mattyp'mat_ml[5,1]);
  local collbl = subinstr("wHO `col1' `col2' `col3' `col4'","s","w",.);
  local rowlbl = "sHO `col1' `col2' `col3' `col4'";
  * disp "`collbl'";
  * disp "`rowlbl'";
  matname `mattyp'mat_ml `collbl', columns(1..5) explicit;
  matname `mattyp'mat_ml `rowlbl', rows(1..5) explicit;
};
* standard error for sHOwHO;
local sdHOHO = 0;
forvalues i = 2/5 {;
  forvalues j = 2/5 {;
    local sdHOHO = `sdHOHO' + semat_ml[`i',`j'];
    };
  };
mat semat_ml[1,1] = sqrt(`sdHOHO');
mat list coefmat_ml; mat list semat_ml;

* Observation-specific extensive margins;
foreach reg in HOM $regagg {;
  local r2dg = substr("`reg'",1,2);
  qui gen double dlta`r2dg' = .;  /* calculate proper derivative later */
  foreach sig in 1 3 5 7 {;
  local sigma`sig' = ln(_b[lnsigma`sig'_`sig':_cons]);
  };
  foreach rho in 1 3 5 7 {;
  local ref = `rho' + 1;
  local rho`rho' = atanh(_b[athrho`rho'_`ref':_cons]);
  };
  capture local mtrlg`r2dg' = sigma`sig'*rho`rho';  /* inverse Mill's ratio is equal to $\rho \times \sigma$*/
  if _rc {;
    local mtrlg`r2dg' = .;
    };
  foreach weff in HOM $regagg {;
    local w2dg = substr("`weff'",1,2);
    local g`r2dg'w`w2dg' = prbdlnw[rownumb(prbdlnw,"i_`reg'"),colnumb(prbdlnw,"w_`weff'")];
    local regvar = "w_othr_`weff'";
    if "`weff'"=="HOM"  local regvar = "w_`weff'";
    * qui gen double xmr`r2dg'w`w2dg' = `g`r2dg'w`w2dg'' * `mtrlg`r2dg'' * dlta`r2dg' * i`r2dg'00 / cf`r2dg' ; /* if log wages used */
    qui gen double xmr`r2dg'w`w2dg' = `g`r2dg'w`w2dg'' * `mtrlg`r2dg'' * dlta`r2dg' * i`r2dg'00 * `regvar' / cf`r2dg' ; /* if plain wages used */
    replace xmr`r2dg'w`w2dg' = 0 if xmr`r2dg'w`w2dg' == .;
    };
  };

* Matrix of mean extensive margin correction terms;
capture mat drop xmrmat_ml;
local rownames : rowfullnames coefmat_ml;  /* Match regional pattern in coefmat at intensive margin */
local colnames : colfullnames coefmat_ml;
local colnum : word count `colnames';
foreach reg in `rownames' {;
  local r2dg = substr("`reg'",-2,2);
  tempname xmat`r2dg'_ml;
  mat `xmat`r2dg''_ml = J(1,`colnum',.);
  matname `xmat`r2dg''_ml s`r2dg', rows(1) explicit;
  local colj = 0;
  foreach weff in `colnames' {;
    local colj = `colj' + 1;
    qui summ xmr`r2dg'`weff' if s`r2dg'~=0 & e(sample), meanonly;  /* only calculate mean over sample MNEs with presence */
    mat `xmat`r2dg''_ml[1,`colj'] = r(mean);
    matname `xmat`r2dg''_ml `weff', columns(`colj') explicit;
    };
  capture mat xmrmat_ml = xmrmat_ml \ `xmat`r2dg''_ml;
  if _rc {;
    mat xmrmat_ml = `xmat`r2dg''_ml;
    };
  };
foreach reg in `rownames' {;
  local r2dg = substr("`reg'",-2,2);
  disp; 
  disp "Lambda Coefficient: Rho times Sigma (= Coefficient on Mill's ratio in `r2dg' in Stata Heckman): `mtrlg`r2dg''";
  disp "Extensive margin correction terms for `r2dg'";
  summ xmr`r2dg'w* if s`r2dg'~=0 & e(sample);
  };
mat list xmrmat_ml;

* Statistics for matrices of shares (predicted shares recommended by Berndt 1991 and Hayashi 2000 but not used);
local eqs = subinstr("`rownames'","sHO","",.);
foreach reg in `eqs' {;
  local r2dg = substr("`reg'",-2,2);
  * qui predict double shat`r2dg', equation(s`r2dg') xb, if e(sample);
  gen double shat`r2dg' = s`r2dg';
  };
gen double shatHO = sHO;
preserve;
collapse (mean) shatHO shatCE shatDE shatOI shatWE if e(sample), fast;
mkmat shatHO shatCE shatDE shatOI shatWE, matrix(Tmat_ml);  /* Tmat to be transposed to a column vector below */
matname Tmat_ml theta, rows(1) explicit;
restore, preserve;
collapse (sd) shatHO shatCE shatDE shatOI shatWE if e(sample), fast;
mkmat shatHO shatCE shatDE shatOI shatWE, matrix(Tmat_sd_ml);
matname Tmat_sd_ml theta_sd, rows(1) explicit;
restore, preserve;
collapse (count) shatHO shatCE shatDE shatOI shatWE if e(sample), fast;
mkmat shatHO shatCE shatDE shatOI shatWE, matrix(Tmat_N_ml);
matname Tmat_N_ml theta_N, rows(1) explicit;
restore, preserve;

* Zeta matrix (r2 in Anderson and Thursby 1986);
foreach sreg in HOM $regagg {;
  local sr2dg = substr("`sreg'",1,2);
  foreach wreg in HOM $regagg {;
    local wr2dg = substr("`wreg'",1,2);
    qui gen double r2s`sr2dg's`wr2dg' = Tmat_N_ml[1,colnumb(Tmat_N_ml,"s`sr2dg'")]^(-3) * ( ( s`sr2dg' - Tmat_ml[1,colnumb(Tmat_ml,"s`sr2dg'")] ) * ( s`sr2dg'*s`wr2dg' - Tmat_ml[1,colnumb(Tmat_ml,"s`sr2dg'")]*Tmat_ml[1,colnumb(Tmat_ml,"s`wr2dg'")] - corrT_translog`usemissg'_ml_tbl[rownumb(corrT_translog`usemissg'_ml_tbl,"s`sr2dg'"),colnumb(corrT_translog`usemissg'_ml_tbl,"s`wr2dg'")]*Tmat_sd_ml[1,colnumb(Tmat_sd_ml,"s`sr2dg'")]*Tmat_sd_ml[1,colnumb(Tmat_sd_ml,"s`wr2dg'")] ) + Tmat_N_ml[1,colnumb(Tmat_N_ml,"s`sr2dg'")]*(Tmat_N_ml[1,colnumb(Tmat_N_ml,"s`sr2dg'")] - 1)*Tmat_ml[1,colnumb(Tmat_ml,"s`sr2dg'")]*Tmat_sd_ml[1,colnumb(Tmat_sd_ml,"s`sr2dg'")]*(Tmat_sd_ml[1,colnumb(Tmat_sd_ml,"s`sr2dg'")] + corrT_translog`usemissg'_ml_tbl[rownumb(corrT_translog`usemissg'_ml_tbl,"s`sr2dg'"),colnumb(corrT_translog`usemissg'_ml_tbl,"s`wr2dg'")]*Tmat_sd_ml[1,colnumb(Tmat_sd_ml,"s`wr2dg'")] ) );
    };
  };
collapse (sum) r2sHOsHO r2sHOsCE r2sHOsDE r2sHOsOI r2sHOsWE r2sCEsHO r2sCEsCE r2sCEsDE r2sCEsOI r2sCEsWE r2sDEsHO r2sDEsCE r2sDEsDE r2sDEsOI r2sDEsWE r2sOIsHO r2sOIsCE r2sOIsDE r2sOIsOI r2sOIsWE r2sWEsHO r2sWEsCE r2sWEsDE r2sWEsOI r2sWEsWE if e(sample), fast;
tempname zetatmp;
mkmat r2sHOsHO r2sHOsCE r2sHOsDE r2sHOsOI r2sHOsWE r2sCEsHO r2sCEsCE r2sCEsDE r2sCEsOI r2sCEsWE r2sDEsHO r2sDEsCE r2sDEsDE r2sDEsOI r2sDEsWE r2sOIsHO r2sOIsCE r2sOIsDE r2sOIsOI r2sOIsWE r2sWEsHO r2sWEsCE r2sWEsDE r2sWEsOI r2sWEsWE, matrix(`zetatmp');
local shnames : colfullnames Tmat_ml;
local shnames = subinstr("`shnames'","hat","",.);
mat zetamat_ml = J(5,5,.);
matname zetamat_ml `shnames', rows(1..5) explicit;
matname zetamat_ml `shnames', columns(1..5) explicit;
foreach sr in `shnames' {;
  local sr2dg = subinstr("`sr'","s","",.);
  foreach wr in `shnames' {;
    local wr2dg = subinstr("`wr'","s","",.);
    mat zetamat_ml[rownumb(zetamat_ml,"`sr'"),colnumb(zetamat_ml,"`wr'")] = `zetatmp'[1,colnumb(`zetatmp',"r2s`sr2dg's`wr2dg'")];
    };
  };
restore;

* Matrices of shares;
mat Tmat_all_ml = Tmat_ml \ Tmat_sd_ml \ Tmat_N_ml;
mat Tmat_ml = Tmat_ml'; /* Tmat transposed to a column vector */
mat Tsqmat_ml = Tmat_ml * Tmat_ml';

* Calculate cross-price elasticities matrix at intensive margin;
mat TotElast_ml = coefmat_ml + xmrmat_ml + Tsqmat_ml;
forvalues r = 2/5 {; /* Fill lower-echelon entries */
local maxcol = `r'-1;
  forvalues m = 1/`maxcol' {;
    mat TotElast_ml[`r',`m'] = TotElast_ml[`r',`m'] / Tmat_ml[`r',1];
    };
  };
forvalues m = 2/5 {; /* Fill upper-echelon entries */
local maxrow = `m'-1;
  forvalues r = 1/`maxrow' {;
    mat TotElast_ml[`r',`m'] = TotElast_ml[`r',`m'] / Tmat_ml[`r',1];
    };
  };
forvalues r = 1/5 {;  /* Fill diagonal entries */
    mat TotElast_ml[`r',`r'] = TotElast_ml[`r',`r'] / Tmat_ml[`r',1] - 1;
  };
matname TotElast_ml `shnames', rows(1..5) explicit;
matname TotElast_ml `shnames', columns(1..5) explicit;
mat list TotElast_ml;
disp;

* Calculate cross-price elasticities matrix for extensive margin;
mat ExtElast_ml = xmrmat_ml + Tsqmat_ml;
forvalues r = 2/5 {; /* Fill lower-echelon entries */
local maxcol = `r'-1;
  forvalues m = 1/`maxcol' {;
    mat ExtElast_ml[`r',`m'] = ExtElast_ml[`r',`m'] / Tmat_ml[`r',1];
    };
  };
forvalues m = 2/5 {; /* Fill upper-echelon entries */
local maxrow = `m'-1;
  forvalues r = 1/`maxrow' {;
    mat ExtElast_ml[`r',`m'] = ExtElast_ml[`r',`m'] / Tmat_ml[`r',1];
    };
  };
forvalues r = 1/5 {;  /* Fill diagonal entries */
    mat ExtElast_ml[`r',`r'] = ExtElast_ml[`r',`r'] / Tmat_ml[`r',1] - 1;
  };
matname ExtElast_ml `shnames', rows(1..5) explicit;
matname ExtElast_ml `shnames', columns(1..5) explicit;
mat list ExtElast_ml;
disp;

* Calculate cross-price elasticities matrix for sum of margins;
mat IntElast_ml = coefmat_ml + Tsqmat_ml;
forvalues r = 2/5 {; /* Fill lower-echelon entries */
local maxcol = `r'-1;
  forvalues m = 1/`maxcol' {;
    mat IntElast_ml[`r',`m'] = IntElast_ml[`r',`m'] / Tmat_ml[`r',1];
    };
  };
forvalues m = 2/5 {; /* Fill upper-echelon entries */
local maxrow = `m'-1;
  forvalues r = 1/`maxrow' {;
    mat IntElast_ml[`r',`m'] = IntElast_ml[`r',`m'] / Tmat_ml[`r',1];
    };
  };
forvalues r = 1/5 {;  /* Fill diagonal entries */
    mat IntElast_ml[`r',`r'] = IntElast_ml[`r',`r'] / Tmat_ml[`r',1] - 1;
  };
matname IntElast_ml `shnames', rows(1..5) explicit;
matname IntElast_ml `shnames', columns(1..5) explicit;
mat list IntElast_ml;
disp;


/* additional concavity checks */
/*-----------------------------*/

* matrix to check (ii) that the cross price elasticities in TotElast sum to zero per row (not per column);
mat elastsum`usemissg'_ml = J(2,5,0);
matname elastsum`usemissg'_ml rowsum colsum, rows(1..2) explicit;
matname elastsum`usemissg'_ml sHO sCE sDE sOI sWE, columns(1..5) explicit;
local i = 0;
foreach sreg in sHO sCE sDE sOI sWE {;
  local i = `i' + 1;
  forvalues j = 1/5 {;
    mat elastsum`usemissg'_ml[1,`i'] = elastsum`usemissg'_ml[1,`i'] + TotElast_ml[`i',`j'];
    mat elastsum`usemissg'_ml[2,`i'] = elastsum`usemissg'_ml[2,`i'] + TotElast_ml[`j',`i'];
    };
  };

* matrix to check (iii) that the AUmat matrices of Allen-Uzawa substitution elasticities are negative semi-definite at each observation;
* calculate MNE-level elasticities (in-sample predictions); 
* set elasticities to zero for regions with no FDI-presence;
local i = 0;
foreach reg1 in HO CE DE OI WE {;
  local i = `i' + 1;
  local j = 0;
  foreach reg2 in HO CE DE OI WE {;
    local j = `j' + 1;
    local coeff = coefmat_ml[`i',`j'];
    if `i' ~= `j' {; 
      qui gen double sigma`reg1's`reg2' = (`coeff' + s`reg1' * s`reg2') / (s`reg1' * s`reg2') if e(sample);
      };
    if `i' == `j' {; 
      qui gen double sigma`reg1's`reg1' = (`coeff' + (s`reg1')^2 - s`reg1') / (s`reg1')^2 if e(sample);
      };      
    qui replace sigma`reg1's`reg2'=0 if sigma`reg1's`reg2'==.;
    label var sigma`reg1's`reg2' "AU elast. empl`reg1' empl`reg2'";
    };
  };
qui count;
local obsnum = `r(N)';
tempname AUmatobs eigenvec eigenval;
mat negeivalcnt`usemissg'_ml = J(1,5,0) \ J(1,5,`obsnum');
matname negeivalcnt`usemissg'_ml r(N_wklyneg_eigval) r(N), rows(1..2) explicit;
matname negeivalcnt`usemissg'_ml wklyneg_1 wklyneg_2 wklyneg_3 wklyneg_4 wklyneg_5, columns(1..5) explicit;
capture forvalues obs = 1/`obsnum'{;
  mat `AUmatobs' = J(5,5,.);
  local i = 0;
  foreach reg1 in HO CE DE OI WE {;
    local i = `i' + 1;
    local j = 0;
    foreach reg2 in HO CE DE OI WE {;
      local j = `j' + 1;
      mat `AUmatobs'[`i',`j'] = sigma`reg1's`reg2'[`obs'];
      };
    };
  matrix symeigen `eigenvec' `eigenval' = `AUmatobs';
  matname `eigenval' eigenvalue, rows(1) explicit;
  mat drop `eigenvec';
  local cntwklyneg = 0;
  forvalues k = 1/5 {;
    if `eigenval'[1,`k'] <= 1.0e-06 {;  /* machine-size weakly negative */
      local cntwklyneg = `cntwklyneg' + 1;
      };
    };
  mat negeivalcnt`usemissg'_ml[1,`cntwklyneg'] = negeivalcnt`usemissg'_ml[1,`cntwklyneg']+1;
  };


/* matrix output */
/*---------------*/

foreach type in b se e {;
  drop _all;
  matsave multiheck`usemissg'_`type', p(estmat/v3ml/pt$par-af$aff-$waglvl-cf) t(double) replace;
  };
matsave negxhatcnt`usemissg'_ml, p(estmat/v3ml/pt$par-af$aff-$waglvl-cf/multiheck`usemissg') dropall t(double) replace; 
matsave elastsum`usemissg'_ml, p(estmat/v3ml/pt$par-af$aff-$waglvl-cf/multiheck`usemissg') dropall t(double) replace; 
matsave negeivalcnt`usemissg'_ml, p(estmat/v3ml/pt$par-af$aff-$waglvl-cf/multiheck`usemissg') dropall t(double) replace; 

mat intelast`usemissg'_ml_b = TotElast_ml;
matsave intelast`usemissg'_ml_b, p(estmat/v3ml/pt$par-af$aff-$waglvl-cf/multiheck`usemissg') dropall t(double) replace;
mat extelast`usemissg'_ml_b = ExtElast_ml;
matsave extelast`usemissg'_ml_b, p(estmat/v3ml/pt$par-af$aff-$waglvl-cf/multiheck`usemissg') dropall t(double) replace;
mat totelast`usemissg'_ml_b = IntElast_ml;
matsave totelast`usemissg'_ml_b, p(estmat/v3ml/pt$par-af$aff-$waglvl-cf/multiheck`usemissg') dropall t(double) replace;

mat vce`usemissg'_ml_tbl = `VCE';
local cols = colsof(vce`usemissg'_ml_tbl);
matsave vce`usemissg'_ml_tbl, p(estmat/v3ml/pt$par-af$aff-$waglvl-cf/multiheck`usemissg') dropall t(double) replace;

mat tmat`usemissg'_ml_tbl = Tmat_all_ml;
matsave tmat`usemissg'_ml_tbl, p(estmat/v3ml/pt$par-af$aff-$waglvl-cf/multiheck`usemissg') dropall t(double) replace;
mat zetamat`usemissg'_ml_tbl = zetamat_ml;
matsave zetamat`usemissg'_ml_tbl, p(estmat/v3ml/pt$par-af$aff-$waglvl-cf/multiheck`usemissg') dropall t(double) replace;

mat coefmat`usemissg'_ml_tbl = coefmat_ml;
matname coefmat`usemissg'_ml_tbl wHO wCE wDE wOI wWE, columns(1..5) explicit;
matname coefmat`usemissg'_ml_tbl sHO sCE sDE sOI sWE, rows(1..5) explicit;
matsave coefmat`usemissg'_ml_tbl, p(estmat/v3ml/pt$par-af$aff-$waglvl-cf/multiheck`usemissg') dropall t(double) replace;

mat semat`usemissg'_ml_tbl = semat_ml;
matname semat`usemissg'_ml_tbl wHO wCE wDE wOI wWE, columns(1..5) explicit;
matname semat`usemissg'_ml_tbl sHO sCE sDE sOI sWE, rows(1..5) explicit;
matsave semat`usemissg'_ml_tbl, p(estmat/v3ml/pt$par-af$aff-$waglvl-cf/multiheck`usemissg') dropall t(double) replace;

mat xmrmat`usemissg'_ml_tbl = xmrmat_ml;
matname xmrmat`usemissg'_ml_tbl wHO wCE wDE wOI wWE, columns(1..5) explicit;
matname xmrmat`usemissg'_ml_tbl sHO sCE sDE sOI sWE, rows(1..5) explicit;
matsave xmrmat`usemissg'_ml_tbl, p(estmat/v3ml/pt$par-af$aff-$waglvl-cf/multiheck`usemissg') dropall t(double) replace;

/*
* clean up;
drop _all;
mat drop coefmat_ml semat_ml xmrmat_ml Tmat_ml Tmat_sd_ml Tmat_N_ml Tmat_all_ml Tsqmat_ml TotElast_ml ExtElast_ml IntElast_ml;
foreach mat in summ_translog`usemissg' corrT_translog`usemissg' vce`usemissg' coefmat`usemissg' semat`usemissg' tmat`usemissg' xmrmat`usemissg' {;
  mat drop `mat'_ml_tbl;
  };
mat drop negxhatcnt`usemissg'_ml elastsum`usemissg'_ml negeivalcnt`usemissg'_ml;
mat drop totelast`usemissg'_ml_b;
mat drop intelast`usemissg'_ml_b;
mat drop extelast`usemissg'_ml_b;
*/

}; /* end of `usemissg' loop */

log close;
cd "$root";

